<?php
/**
 * The Agora script to delete a forum.
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * See the enclosed file COPYING for license information (GPL). If you did not
 * receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * $Horde: agora/deleteforum.php,v 1.23 2005/01/03 14:34:45 jan Exp $
 */

@define('AGORA_BASE', dirname(__FILE__));
require_once AGORA_BASE . '/lib/base.php';
require_once AGORA_BASE . '/lib/Forums.php';
require_once 'Horde/Form.php';
require_once 'Horde/Form/Renderer.php';
require_once 'Horde/Variables.php';

/* Only admin should be using this. */
if (!Auth::isAdmin('agora:admin')) {
    Horde::authenticationFailureRedirect();
}

/* Set up the forums object. */
$forums = &Agora_Forums::singleton();

$vars = &Variables::getDefaultVariables();
$formname = $vars->get('formname');
list($forum_id) = Agora::getAgoraId();

/* Get the forum to be deleted. */
if ($forum_id && !$formname) {
    $forum = &$forums->getForum($forum_id);
    $vars = &new Variables($forum->getForumArray());
}

/* Get the form object. */
$form = &Horde_Form::singleton('', $vars, _("Delete Forum"));

/* Set up the form. */
$form->setButtons(array(_("Delete"), _("Do not delete")));
$form->addHidden('', 'forum_id', 'int', false);
$form->addVariable(_("This will delete the forum, any subforums and all relative messages."), 'prompt', 'description', false);
$form->addVariable(_("Forum name"), 'forum_name', 'text', false, true);

/* Get a list of available forums. */
$forums_list = Agora::formatCategoryTree($forums->getForums($forum_id, false, null, null, true));
if (!empty($forums_list)) {
    $html = implode('<br />', $forums_list);
    $form->addVariable(_("Subforums"), 'subforums', 'html', false, true);
    $vars->set('subforums', $html);
}

if ($vars->get('submitbutton') == _("Delete")) {
    if ($form->validate($vars)) {
        $form->getInfo($vars, $info);

        /* Delete forum. */
        $del_forum = $forums->deleteForum($info['forum_id']);
        if (is_a($del_forum, 'PEAR_Error')) {
            $notification->push(sprintf(_("Could not delete the forum. %s"), $del_forum->message), 'horde.error');
        } else {
            $notification->push(_("Forum deleted."), 'horde.success');
            $url = Horde::applicationUrl('forums.php', true);
            header('Location: ' . $url);
            exit;
        }
    }
} elseif ($vars->get('submitbutton') == _("Do not delete")) {
    $notification->push(_("Forum not deleted."), 'horde.message');
    $url = Horde::applicationUrl('forums.php');
    header('Location: ' . $url);
    exit;
}

/* Render the form. */
$renderer = &new Horde_Form_Renderer();
$main = Util::bufferOutput(array($form, 'renderActive'), $renderer, $vars, 'deleteforum.php', 'post');

/* Set up the Horde_Template variables. */
$template = &new Horde_Template();
$template->set('menu', Agora::getMenu('string'));
$template->set('main', $main);
$template->set('notify', Util::bufferOutput(array($notification, 'notify'), array('listeners' => 'status')));

require AGORA_TEMPLATES . '/common-header.inc';
echo $template->fetch(AGORA_TEMPLATES . '/main/main.html');
require $registry->get('templates', 'horde') . '/common-footer.inc';
