<?php
/**
 * The Agora script to create or edit a forum.
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * See the enclosed file COPYING for license information (GPL). If you did not
 * receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * $Horde: agora/editforum.php,v 1.28 2005/01/03 14:34:45 jan Exp $
 */

@define('AGORA_BASE', dirname(__FILE__));
require_once AGORA_BASE . '/lib/base.php';
require_once AGORA_BASE . '/lib/Forums.php';
require_once 'Horde/Form.php';
require_once 'Horde/Form/Renderer.php';
require_once 'Horde/Variables.php';

/* Only admin should be using this. */
if (!Auth::isAdmin('agora:admin')) {
    Horde::authenticationFailureRedirect();
}

/* Set up the forums object. */
$forums = &Agora_Forums::singleton();

$vars = &Variables::getDefaultVariables();
list($forum_id) = Agora::getAgoraId();
$title = $forum_id ? _("Edit Forum") : _("New Forum");

/* Get the form object. */
$form = &Horde_Form::singleton('', $vars, $title);

/* Set up the form. */
$form->setButtons($forum_id ? _("Update") : _("Create"), true);
$form->addHidden('', 'forum_id', 'int', false);
$form->addVariable(_("Forum name"), 'forum_name', 'text', true);

/* Get a list of available forums. */
$forums_list = Agora::formatCategoryTree($forums->getForums(DATATREE_ROOT, false));
$form->addVariable(_("Parent forum"), 'forum_parent_id', 'enum', false, false, null, array($forums_list));
$form->addVariable(_("Enter a brief description of this forum"), 'forum_description', 'longtext', true, false, null, array(4, 40));
$form->addVariable(_("Is this a moderated forum?"), 'forum_moderated', 'boolean', false, false, _("Set this if you want all messages to be checked by a moderator before they are posted."));
if ($conf['forums']['enable_attachments'] == '0') {
    $form->addVariable(_("Allow attachments in this forum?"), 'forum_attachments', 'boolean', false, false, _("If selected users will be able to attach files to their messages."));
}

if ($form->validate($vars)) {
    $form->getInfo($vars, $info);

    /* Store this forum object and get back a new forum_id. */
    $forum_id = $forums->saveForum($info);
    if (is_a($forum_id, 'PEAR_Error')) {
        $notification->push(sprintf(_("Could not create the forum. %s"), $forum_id->message), 'horde.error');
    } else {
        $notification->push(!empty($info['forum_id']) ? _("Forum Modified") : _("Forum created."), 'horde.success');
        $url = Agora::setAgoraId($forum_id, null, Horde::applicationUrl('threads.php', true));
        header('Location: ' . $url);
        exit;
    }
}

/* Check if a forum is being edited. */
if ($forum_id) {
    $forum = &$forums->getForum($forum_id);
    $vars = &new Variables($forum->getForumArray());
}

/* Render the form. */
$renderer = &new Horde_Form_Renderer();
$main = Util::bufferOutput(array($form, 'renderActive'), $renderer, $vars, 'editforum.php', 'post');

/* Set up the Horde_Template variables. */
$template = &new Horde_Template();
$template->set('menu', Agora::getMenu('string'));
$template->set('main', $main);
$template->set('notify', Util::bufferOutput(array($notification, 'notify'), array('listeners' => 'status')));

require AGORA_TEMPLATES . '/common-header.inc';
echo $template->fetch(AGORA_TEMPLATES . '/main/main.html');
require $registry->get('templates', 'horde') . '/common-footer.inc';
