<?php
/**
 * The Agora script to display a list of forums.
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * $Horde: agora/forums.php,v 1.39 2005/01/03 14:34:45 jan Exp $
 *
 * See the enclosed file COPYING for license information (GPL). If you did not
 * receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

@define('AGORA_BASE', dirname(__FILE__));
require_once AGORA_BASE . '/lib/base.php';
require_once AGORA_BASE . '/lib/Forums.php';
require_once 'Horde/Variables.php';
require_once 'Horde/UI/Pager.php';

/* Set up the forums object. */
$forums = &Agora_Forums::singleton();

/* Get the sorting. */
$sort_by = Agora::getSortBy('forums');
$sort_dir = Agora::getSortDir('forums');

/* Which forums page are we on?  Default to page 0. */
$forum_page = Util::getFormData('forum_page', 0);
$forums_per_page = $prefs->getValue('forums_per_page');
$forum_start = $forum_page * $forums_per_page;

/* Get the list of forums. */
$forums_list = $forums->getForums(DATATREE_ROOT, true, $sort_by, $sort_dir, true);
if (empty($forums_list)) {
    $forums_count = 0;
    $notification->push(_("No forums available."), 'horde.warning');
} else {
    $forums_count = count($forums_list);
    $forums_list = array_slice($forums_list, $forum_start, $forums_per_page);
}

/* Set up the column headers. */
$col_headers = array(array('forum_name' => _("Forum"), 'flat_forum_name' => _("Flat")), 'forum_desc' => _("Description"), 'message_count' => _("Posts"), 'last_message_subject' => _("Last Post"), 'last_message_author' => _("Posted By"), 'last_message_date' => _("Date"));
$col_headers = Agora::formatColumnHeaders($col_headers, $sort_by, $sort_dir, 'forums');

/* Set up the template tags. */
$template = &new Horde_Template();
$template->setOption('gettext', true);
$template->set('col_headers', $col_headers);
$template->set('forums_list', $forums_list, true);
$template->set('menu', Agora::getMenu('string'));
$template->set('notify', Util::bufferOutput(array($notification, 'notify'), array('listeners' => 'status')));

/* Set up pager. */
$vars = &Variables::getDefaultVariables();
$pager_ob = &new Horde_UI_Pager('forum_page', $vars, array('num' => $forums_count, 'url' => 'forums.php', 'perpage' => $forums_per_page));
$template->set('pager_link', $pager_ob->render(), true);

$title = _("All Forums");
require AGORA_TEMPLATES . '/common-header.inc';
echo $template->fetch(AGORA_TEMPLATES . '/forums/forums.html');
require $registry->get('templates', 'horde') . '/common-footer.inc';
