<?php

$block_name = _("Forums");

/**
 * Agora's Forum Block Class
 *
 * This file provides a list of Agora forums through the Horde_Blocks, by
 * extending the Horde_Blocks class.
 *
 * $Horde: agora/lib/Block/forums.php,v 1.21 2005/01/03 14:34:46 jan Exp $
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * See the enclosed file COPYING for license information (GPL). If you did not
 * receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Marko Djukic <marko@oblo.com>
 * @version $Revision $
 * @package Horde_Block
 */
class Horde_Block_agora_forums extends Horde_Block {

    var $_app = 'agora';

    function _title()
    {
        return Horde::link(Horde::applicationUrl('forums.php', true), _("Forums"), 'header') . _("Forums") . '</a>';
    }

    function _params()
    {
        /* Display the last X number of threads. */
        $forum_display = array();
        $forum_display['name'] = _("Only display this many forums (0 to display all forums)");
        $forum_display['type'] = 'int';
        $forum_display['default'] = 0;
        $forum_display['values'] = $GLOBALS['prefs']->getValue('forums_block_display');

        return array('forum_display' => $forum_display);
    }

    function _content()
    {
        @define('AGORA_BASE', dirname(__FILE__) . '/../..');
        require_once AGORA_BASE . '/lib/Forums.php';
        require_once 'Horde/Template.php';

        /* Set up the forums object. */
        $forums = &Agora_Forums::singleton();

        /* Get the sorting. */
        $sort_by = Agora::getSortBy('forums');
        $sort_dir = Agora::getSortDir('forums');

        /* Get the list of forums. */
        $forums_list = $forums->getForums(DATATREE_ROOT, true, $sort_by, $sort_dir, true);

        /* Show a message if no available forums. Don't raise an error
         * as it is not an error to have no forums. */
        if (empty($forums_list)) {
            return _("No available forums.");
        }

        /* Display only the most recent threads if preference set. */
        if (!empty($this->_params['forum_display'])) {
            $forums_list = array_slice($forums_list, 0, $this->_params['forum_display']);
        }

        /* Set up the column headers. */
        $col_headers = array('forum_name' => _("Forum"), 'message_count' => _("Posts"), 'last_message_subject' => _("Last Post"), 'last_message_author' => _("Posted By"), 'last_message_date' => _("Date"));
        $col_headers = Agora::formatColumnHeaders($col_headers, $sort_by, $sort_dir, 'forums');

        /* Set up the template tags. */
        $template = &new Horde_Template();
        $template->setOption('gettext', true);
        $template->set('col_headers', $col_headers);
        $template->set('forums_list', $forums_list, true);

        return $template->fetch($GLOBALS['registry']->get('templates') . '/block/forums.html');
    }

}
