<?php

$block_name = _("Single Thread");

/**
 * Agora Forum Thread Block Class
 *
 * This file provides an api to include an Agora forum's thread into any other
 * Horde app through the Horde_Blocks, by extending the Horde_Blocks class.
 *
 * $Horde: agora/lib/Block/thread.php,v 1.14 2005/01/03 14:34:46 jan Exp $
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * See the enclosed file COPYING for license information (GPL). If you did not
 * receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Marko Djukic <marko@oblo.com>
 * @version $Revision: 1.14 $
 * @package Horde_Block
 */
class Horde_Block_agora_thread extends Horde_Block {

    var $_threads = array();
    var $_app = 'agora';

    function Horde_Block_agora_thread($params = array())
    {
        @define('AGORA_BASE', dirname(__FILE__) . '/../..');

        parent::Horde_Block($params);

        /* If there's a forum id get the messages. */
        if (isset($this->_params['forum_id'])) {
            require_once AGORA_BASE . '/lib/Messages.php';
            $this->_threads = &Agora_Messages::singleton(array('forum_id' => $this->_params['forum_id']));
        }
    }

    function _params()
    {
        require_once AGORA_BASE . '/lib/Forums.php';

        $forumOb = &Agora_Forums::singleton();
        $forums_list = $forumOb->getForums(DATATREE_ROOT, true, 'forum_name', 0, true);

        $threads = array('name'   => _("Thread"),
                         'type'   => 'mlenum',
                         'values' => array());
        foreach ($forums_list as $forum_id => $forum) {
            $threadsOb = &Agora_Messages::singleton(array('forum_id' => $forum_id));
            $threads_list = $threadsOb->getThreads($forum_id);
            foreach ($threads_list as $thread_id => $thread) {
                if (!isset($threads['default'])) {
                    $threads['default'] = $forum_id . '.' . $thread['message_id'];
                }
                $threads['values'][$forum['indent'] . $forum['forum_name']][$forum_id . '.' . $thread['message_id']] = $thread['plain_subject'];
            }
        }

        return array('thread_id' => $threads);
    }

    function _title()
    {
        /* Simpler title if no valid threads. */
        if (is_a($this->_threads, 'PEAR_Error')) {
            return _("Single Thread");
        }

        return _("Thread");
    }

    function _content()
    {
        /* Return empty if we don't have a thread set. */
        if (empty($this->_params['thread_id'])) {
            return '';
        }

        require_once AGORA_BASE . '/lib/Messages.php';
        require_once 'Horde/Template.php';

        list($forum_id, $message_id) = explode('.', $this->_params['thread_id']);
        $params = array('message_id' => $message_id,
                        'forum_id' => $forum_id);

        /* Set up the message object. */
        $messages = &Agora_Messages::singleton($params);

        /* Check if valid thread, otherwise show forum list. */
        if (is_a($messages, 'PEAR_Error') || empty($messages)) {
            return PEAR::raiseError(_("Unable to fetch selected thread."));
        }

        /* Get the sorting. */
        $sort_by = Agora::getSortBy('threads');
        $sort_dir = Agora::getSortDir('threads');

        /* Get the message array and the sorted thread list. */
        $threads_list = $messages->getThreads($messages->getThreadRoot($forum_id, $message_id), true, $sort_by, $sort_dir, true, Horde::selfUrl());

        /* Set up the column headers. */
        $col_headers = array(array('message_thread' => _("Thread"), 'message_subject' => _("Subject")), 'message_author' => _("Posted By"), 'message_date' => _("Date"));
        $col_headers = Agora::formatColumnHeaders($col_headers, $sort_by, $sort_dir, 'threads');

        /* Set up the template tags. */
        $template = &new Horde_Template();
        $template->set('col_headers', $col_headers);
        $template->set('threads_list', $threads_list, true);
        $template->set('threads_list_header', _("Thread List"));
        $template->set('thread_view_bodies', $GLOBALS['prefs']->getValue('thread_view_bodies'), true);

        return $template->fetch($GLOBALS['registry']->get('templates') . '/block/thread.html');
    }

}
