<?php

$block_name = _("Threads");

/**
 * Agora Forum Thread Block Class
 *
 * This file provides an api to include an Agora forum's thread into any other
 * Horde app through the Horde_Blocks, by extending the Horde_Blocks class.
 *
 * $Horde: agora/lib/Block/threads.php,v 1.39 2005/01/03 14:34:46 jan Exp $
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * See the enclosed file COPYING for license information (GPL). If you did not
 * receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Marko Djukic <marko@oblo.com>
 * @version $Revision: 1.39 $
 * @package Horde_Block
 */
class Horde_Block_agora_threads extends Horde_Block {

    var $_threads = array();
    var $_app = 'agora';

    function Horde_Block_agora_threads($params = array())
    {
        @define('AGORA_BASE', dirname(__FILE__) . '/../..');

        parent::Horde_Block($params);

        /* If there's a forum id get the messages. */
        if (isset($this->_params['forum_id'])) {
            require_once AGORA_BASE . '/lib/Messages.php';
            $this->_threads = &Agora_Messages::singleton(array('forum_id' => $this->_params['forum_id']));
        }
    }

    function _params()
    {
        require_once AGORA_BASE . '/lib/Forums.php';

        /* Get the list of forums to display. */
        $forumOb = &Agora_Forums::singleton();
        $forums_list = $forumOb->getForums(DATATREE_ROOT, true, 'forum_name', 0, true);

        $forum_id = array();
        $forum_id['name'] = _("Forum");
        $forum_id['type'] = 'enum';
        foreach ($forums_list as $id => $forum) {
            if (!isset($forum_id['default'])) {
                $forum_id['default'] = $id;
            }
            $forum_id['values'][$id] = $forum['indent'] . $forum['forum_name'];
        }

        /* Display the last X number of threads. */
        $thread_display = array();
        $thread_display['name'] = _("Only display this many threads (0 to display all threads)");
        $thread_display['type'] = 'int';
        $thread_display['default'] = 0;
        $thread_display['values'] = $GLOBALS['prefs']->getValue('threads_block_display');
 
        return array('forum_id' => $forum_id, 'thread_display' => $thread_display);
    }

    function _title()
    {
        global $registry;

        /* Simpler title if no valid threads. */
        if (is_a($this->_threads, 'PEAR_Error')) {
            return _("Threads");
        }

        $forum_array = $this->_threads->forum->getForumArray();
        $title = sprintf(_("Threads in \"%s\""), $forum_array['forum_name']);
        $html = Horde::link(Agora::setAgoraId($this->_params['forum_id'], null, Horde::applicationUrl('threads.php', true)), $title, 'header') . $title . '</a>';

        if (Auth::getAuth()) {
            $url = Agora::setAgoraId($this->_params['forum_id'], null, Horde::applicationUrl('messages/edit.php'));
            $html .= ' :: ' . Horde::link($url, _("Post message"), 'smallheader') . Horde::img('newmessage.png', _("Post message")) . ' ' . _("Post message") . '</a>';
        }

        return $html;
    }

    function _content()
    {
        require_once AGORA_BASE . '/lib/Messages.php';
        require_once 'Horde/Template.php';

        /* Get the sorting. */
        $sort_by = Agora::getSortBy('threads');
        $sort_dir = Agora::getSortDir('threads');

        $forum_id = $this->_params['forum_id'];
        $threads = &$this->_threads;

        /* Check if valid thread, otherwise show forum list. */
        if (is_a($threads, 'PEAR_Error')) {
            return PEAR::raiseError(_("Unable to fetch threads for selected forum."));
        }

        /* Get a list of threads. */
        $threads_list = $threads->getThreads($forum_id, false, $sort_by, $sort_dir, false, Horde::selfURL());

        /* Show a message if no available threads. Don't raise an error
         * as it is not an error to have no threads. */
        if (empty($threads_list)) {
            return _("No available threads.");
        }

        /* Display only the most recent threads if preference set. */
        if (!empty($this->_params['thread_display'])) {
            $threads_list = array_slice($threads_list, 0, $this->_params['thread_display']);
        }

        /* Set up the column headers. */
        $col_headers = array('message_subject' => _("Subject"), 'message_author' => _("Posted By"), 'message_date' => _("Date"));
        $col_headers = Agora::formatColumnHeaders($col_headers, $sort_by, $sort_dir, 'threads');

        /* Set up the template tags. */
        $template = &new Horde_Template();
        $template->set('col_headers', $col_headers);
        $template->set('threads', $threads_list, true);

        return $template->fetch($GLOBALS['registry']->get('templates') . '/block/threads.html');
    }

}
