<?php
/**
 * Agora_Messages:: provides the functions to access both threads and
 * individual messages.
 *
 * $Horde: agora/lib/Messages.php,v 1.111 2005/01/03 14:34:46 jan Exp $
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * See the enclosed file COPYING for license information (GPL). If you did not
 * receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Marko Djukic <marko@oblo.com>
 * @version $Revision: 1.111 $
 * @package Agora
 */
class Agora_Messages {

    var $threads;
    var $forum;

    var $_params;

    var $_sortdir = 0;

    /**
     * Constructor
     */
    function Agora_Messages($params)
    {
        $this->_params = $params;

        global $conf;
        if (empty($conf['datatree']['driver'])) {
            Horde::fatal(_("You must configure a Horde Categories backend to use Agora."), __FILE__, __LINE__);
        }
        $driver = $conf['datatree']['driver'];
        $dt_params = array_merge(Horde::getDriverConfig('datatree', $driver),
                                 array('group' => 'agora.threads'));

        /* Connect to the backend for tracking forums. */
        $this->threads = &DataTree::singleton($driver, $dt_params);
    }

    function &factory($params)
    {
        $messages = &new Agora_Messages($params);

        if (!empty($params['forum_id'])) {
            $success = Agora_Messages::setupForumObject($messages, $params);
            /* Check if there was a valid forum object to get. */
            if (is_a($success, 'PEAR_Error')) {
                return $success;
            }
        }

        return $messages;
    }

    function &singleton($params = array())
    {
        static $messages = array();

        $signature = serialize($params);
        if (!isset($messages[$signature])) {
            $messages[$signature] = &Agora_Messages::factory($params);
        }

        return $messages[$signature];
    }

    function &setupForumObject(&$messages, $params)
    {
        /* Set up the forum object. */
        require_once AGORA_BASE . '/lib/Forums.php';
        $data_root = (!empty($params['data_root'])) ? $params['data_root'] : 'agora.forums';
        $forums = &Agora_Forums::singleton($data_root);
        $messages->forum = &$forums->getForum($params['forum_id']);

        /* Check if valid forum. */
        if (is_a($messages->forum, 'PEAR_Error')) {
            return $messages->forum;
        }
    }

    function &newMessage($message_path)
    {
        if (empty($message_path)) {
            return PEAR::raiseError(_("Can not create a message without a message thread."));
        }
        $message = &new DataTreeObject_Message($message_path);
        $message->setMessagesOb($this);
        return $message;
    }

    /**
     * Check if attachments are allowed in messages for the current forum.
     *
     * @access public
     *
     * @return bool  Whether attachments allowed or not.
     */
    function allowAttachments()
    {
        global $conf;
        if ($conf['forums']['enable_attachments'] == '1' ||
            ($conf['forums']['enable_attachments'] == '0' && !empty($this->forum->data['forum_attachments']))) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Save the message.
     *
     * @access public
     *
     * @param array $info  Array containing all the message data to save.
     *
     * @return mixed  Message ID on success or PEAR_Error on failure.
     */
    function saveMessage(&$info)
    {
        /* Check if message_id exists to figure out if updating or creating
         * a new message. */
        if (empty($info['message_id'])) {
            /* Increment the message sequential id for this forum and save. */
            $this->forum->data['message_seq']++;
            $this->forum->saveForumData();

            /* Work out the message thread. */
            if (!empty($info['message_parent_id'])) {
                /* There's a parent message so use that as the thread base. */
                $message_thread = $this->threads->getName($info['message_parent_id']);
            } else {
                /* No parent, so start with the forum_id. */
                $message_thread = $info['forum_id'];
            }

            /* Add sequential id for this message to the thread path. */
            $message_thread .= ':' . $info['forum_id'] . '-' . $this->forum->data['message_seq'];

            /* Create the new message object. */
            $message = &$this->newMessage($message_thread);
        } else {
            /* Editing so fetch the existing message object. */
            $message = &$this->getMessage($info['message_id']);
        }

        $message->data['message_subject'] = $info['message_subject'];
        $message->data['message_body']    = $info['message_body'];
        $message->data['forum_id']        = $info['forum_id'];
        if ($this->forum->isModerated()) {
            $message->data['moderate'] = true;
        }

        /* Handle attachment saves or deletions. */
        if (!empty($info['message_attachment']) ||
            !empty($info['attachment_delete'])) {
            if (is_a($vfs = Agora::getVFS(), 'PEAR_Error')) {
                return $vfs;
            }
            $vfs_dir = AGORA_VFS_PATH . $info['forum_id'] . '/' . $info['message_id'];

            /* Check if delete requested or new attachment loaded, and delete
             * any existing one. */
            if (!empty($info['attachment_delete']) ||
                (!empty($message->data['message_attachment']) &&
                $vfs->exists($vfs_dir, $message->data['message_attachment']['name']))) {
                $delete = $vfs->deleteFile($vfs_dir, $message->data['message_attachment']['name']);
                if (is_a($delete, 'PEAR_Error')) {
                    return $delete;
                }
            }

            /* Check what to do with attachment information fields. */
            if (!empty($info['message_attachment'])) {
                /* Save new attachment information. */
                $message->data['message_attachment'] = array(
                    'name' => $info['message_attachment']['name'],
                    'type' => $info['message_attachment']['type'],
                    'size' => $info['message_attachment']['size']);
            } else {
                /* Delete existing attachment information. */
                unset ($message->data['message_attachment']);
            }
        }

        if (empty($info['message_id'])) {
            /* New message. First add timestamp and author. */
            $message->data['message_timestamp'] = time();
            if (Auth::getAuth()) {
                $message->data['message_author'] = Auth::getAuth();
            } else {
                $message->data['message_author'] = $info['posted_by'];
            }

            /* Add the datatree object. */
            $result = $this->threads->add($message);
            $info['message_id'] = $this->threads->getId($message);
        } else {
            $result = $this->threads->updateData($message);
        }

        /* Save any attachment if available to VFS. */
        if (!empty($info['message_attachment'])) {
            /* Save the new attachment. */
            $result = $vfs->write($vfs_dir, $info['message_attachment']['name'], $info['message_attachment']['file'], true);
            if (is_a($result, 'PEAR_Error')) {
                return $result;
            }
        }

        return $info['message_id'];
    }

    /**
     * Fetches a message.
     *
     * @public
     *
     * @param integer $message_id  The ID of the message to fetch.
     */
    function &getMessage($message_id)
    {
        $message = &$this->threads->getObjectById($message_id, 'DataTreeObject_Message');
        if (is_a($message, 'PEAR_Error')) {
            return $message;
        }
        $message->setMessagesOb($this);
        return $message;
    }

    /**
     * Returns a hash with all information necessary to reply to a message.
     *
     * @param integer $message_id  The ID of the parent message to reply to.
     *
     * @return array  A hash with all relevant information.
     */
    function replyMessage($message_id)
    {
        global $prefs;

        $parent_message = &$this->getMessage($message_id);
        if (is_a($parent_message, 'PEAR_Error')) {
            return $parent_message;
        }

        /* Get the message array if this is a valid parent message. */
        $message_array = $parent_message->getMessageArray();

        /* Set up the form subject with the parent subject. */
        if (String::lower(String::substr($message_array['plain_subject'], 0, 3)) != 're:') {
            $message_array['plain_subject'] = 'Re: ' . $message_array['plain_subject'];
        }
        $message_array['plain_body'] = String::wrap($message_array['plain_body'], 60, "\n> ", NLS::getCharset());
        if (!empty($message_array['plain_body'])) {
            $message_array['plain_body'] = '> ' . $message_array['plain_body'] . "\n";
        }

        /* Set up the tags to display the parent message below. */
        $message_array['message_subject'] = sprintf(_("Replying to \"%s\""), $message_array['message_subject']);
        $message_array['message_from'] = sprintf(_("Posted by %s on %s"), htmlspecialchars($message_array['message_author']), strftime($prefs->getValue('date_format'), $message_array['message_timestamp']));

        return $message_array;
    }

    function getMessageId($message_thread)
    {
        return $this->threads->getId($message_thread);
    }

    /**
     * Deletes a message and all replies.
     *
     * @public
     *
     * @param integer $message_id  The ID of the message to delete.
     */
    function deleteMessage($message_id)
    {
        $message = $this->getMessage($message_id);
        $message_id = $this->getMessageId($message->name);
        /* Check if delete requested or new attachment loaded, and delete
         * any existing one. */
        if (!empty($message->data['message_attachment'])) {
            $vfs_dir = AGORA_VFS_PATH . Agora::getForumId($message->name) . '/' . $message_id;
            $delete = $vfs->deleteFile($vfs_dir, $message->data['message_attachment']['name']);
            if (is_a($delete, 'PEAR_Error')) {
                return $delete;
            }
        }

        return $this->threads->remove($this->threads->getName($message_id), true);
    }

    /**
     * Deletes an entire message thread.
     *
     * @public
     *
     * @param integer $message_id  The ID of the message to delete. If not
     *                             specified will delete all the threads for the
     *                             current forum.
     */
    function deleteThread($message_id = null)
    {
        if ($message_id) {
            /* Delete thread for specific message ID. */
            $root = $this->getThreadRoot($message_id);
            return $this->threads->remove($root, true);
        } else {
            /* If no thread has been created just return true. */
            if (!$this->threads->exists($this->_params['forum_id'])) {
                return true;
            }
            /* Delete all threads for current forum. */
            return $this->threads->remove($this->_params['forum_id'], true);
        }
    }

    /**
     * Returns a list of threads.
     *
     * @public
     *
     * @param integer $thread_root   Message at which to start the thread.
     * @param boolean $all_levels    Show all child levels or just one level.
     * @param string  $sort_by       The column by which to sort.
     * @param integer $sort_dir      The direction by which to sort:
     *                                   0 - ascending
     *                                   1 - descending
     * @param boolean $message_view
     * @param string  $link_back     A url to pass to the reply script which
     *                               will be returned to after an insertion of a
     *                               post. Useful in cases when this thread view
     *                               is used in blocks to return to the original
     *                               page rather than to Agora.
     * @param string  $reply_url     An alternative URL where the reply link
     *                               points to. Takes "%p" as a placeholder for
     *                               parent message ID.
     */
    function &getThreads($thread_root, $all_levels = false,
                         $sort_by = 'message_timestamp', $sort_direction = 0,
                         $message_view = false, $link_back = '',
                         $reply_url = null)
    {
        global $registry;

        $threads_list = $this->threads->get(DATATREE_FORMAT_FLAT, $thread_root, true, DATATREE_ROOT, $all_levels ? -1 : 1, true);
        if (is_a($threads_list, 'PEAR_Error')) {
            Horde::logMessage($threads_list, __FILE__, __LINE__, PEAR_LOG_ERR);
            $threads_list = array();
        }

        $threads_list = $this->threads->getObjects(array_keys($threads_list), 'DataTreeObject_Message');
        if (empty($threads_list)) {
            return array();
        }

        $threads_array = array();

        /* Set up the base urls for actions. */
        $cparts = explode(':', $thread_root);
        $forum_id = array_shift($cparts);
        if ($message_view) {
            $view_url = Horde::applicationUrl('messages/message.php');
        } else {
            $view_url = Horde::applicationUrl('messages/index.php');
        }
        $edit_url = Horde::applicationUrl('messages/edit.php');
        $del_url  = Horde::applicationUrl('messages/delete.php');

        /* Loop through the threads and set up the array. */
        $thread_ids = array_keys($threads_list);
        foreach ($thread_ids as $i => $thread_id) {
            $thread = $threads_list[$thread_id];
            $message_id = $this->threads->getId($thread_id);
            $message = &$this->getMessage($message_id);
            /* Skip any messages with errors or that need to be moderated. */
            if (is_a($message, 'PEAR_Error') || $message->needsModerating()) {
                continue;
            }

            $threads_array[$i] = $message->getMessageArray();
            $threads_array[$i]['message_name'] = $thread->name;
            $threads_array[$i]['message_author'] = htmlspecialchars($threads_array[$i]['message_author']);
            $threads_array[$i]['message_body'] = Text_Filter::filter($threads_array[$i]['message_body'], 'highlightquotes');
            $threads_array[$i]['message_attachment'] = $message->getAttachmentLink();

            if (!empty($this->_params['message_id']) && $message_id == $this->_params['message_id']) {
                $threads_array[$i]['class'] = 'selected';
            }

            /* Set up indenting for threads. */
            if ($sort_by == 'message_thread') {
                $indent = explode(':', $message->getName());
                $threads_array[$i]['indent'] = count($indent) - 2;
                array_pop($indent);
                $threads_array[$i]['parent'] = count($indent) > 1 ? implode(':', $indent) : null;

            }

            $threads_array[$i]['message_date'] = strftime($GLOBALS['prefs']->getValue('date_format'), $message->data['message_timestamp']);

            /* Link to view the message. */
            $url = Agora::setAgoraId($forum_id, $message_id, $view_url);
            $threads_array[$i]['link'] = Horde::link($url, $message->data['message_subject'], '', '', '', $message->data['message_subject']);

            /* Button to post a reply to the message. */
            if (!$message->isLocked()) {
                if ($reply_url) {
                    $url = $reply_url;
                } else {
                    $url = Agora::setAgoraId($forum_id, null, $edit_url);
                    $url = Util::addParameter($url, 'message_parent_id', '%p');
                    $url = str_replace('%25', '%', $url);
                }
                $url = str_replace('%p', $message_id, $url);
                if (!empty($link_back)) {
                    $url = Util::addParameter($url, 'url', $link_back);
                }
                $threads_array[$i]['reply'] = Horde::link($url, _("Reply to message"), '', '', '', _("Reply to message")) . Horde::img('reply.png', _("Reply to message")) . '</a>';
            }

            if (Auth::isAdmin('agora:admin')) {
                /* Button to edit the message. */
                $url = Agora::setAgoraId($forum_id, $message_id, $edit_url);
                $threads_array[$i]['actions'][] = Horde::link($url, _("Edit message"), '', '', '', _("Edit message")) . Horde::img('edit.png', _("Edit message"), '', $registry->getImageDir('horde')) . '</a>';

                /* Button to delete the message. */
                $url = Agora::setAgoraId($forum_id, $message_id, $del_url);
                $threads_array[$i]['actions'][] = Horde::link($url, _("Delete message"), '', '', '', _("Delete message")) . Horde::img('delete.png', _("Delete message"), null, $registry->getImageDir('horde')) . '</a>';
            }

            /* Hot thread icon. */
            if ($message->isHot() && empty($cparts)) {
                $threads_array[$i]['actions'][] = Horde::img('hot.png', _("Hot thread"));
            }
        }

        /* Check if sorting by date, then actually sort by timestamp. */
        if ($sort_by == 'message_date') {
            $sort_by = 'message_timestamp';
        }

        /* Do the sort. */
        if ($sort_by == 'message_thread') {
            $this->_sortdir = $sort_direction;
            usort($threads_array, array($this, '_sortByThread'));
        } else {
            require_once 'Horde/Array.php';
            Horde_Array::arraySort($threads_array, $sort_by, $sort_direction, false);
        }

        /* Set row classes. */
        for ($i = 0; $i < count($threads_array); $i++) {
            if (!isset($threads_array[$i]['class'])) {
                $threads_array[$i]['class'] = 'item' . (($i + 1) % 2);
            }
        }

        return $threads_array;
    }

    /**
     * Fetches a list of messages awaiting moderation. Selects all
     * messages, irrespective of the thread root (starts from
     * DATATREE_ROOT in the 'agora.threads' of the DataTree group),
     * which have the 'moderate' flag set in the attributes.
     *
     * @access public
     *
     * @param string  $sort_by       The column by which to sort.
     * @param integer $sort_dir      The direction by which to sort:
     *                                   0 - ascending
     *                                   1 - descending
     */
    function getModerateList($sort_by, $sort_direction)
    {
        global $registry, $forums;

        /* Set up the selection criteria. */
        $criteria['AND'] = array('AND' =>
                                array('field' => 'name',
                                      'op'    => '=',
                                      'test'  => 'message'),
                                array('field' => 'key',
                                      'op'    => '=',
                                      'test'  => 'moderate'));
        $messages_list = $this->threads->getByAttributes($criteria, DATATREE_ROOT);
        if (is_a($messages_list, 'PEAR_Error')) {
            Horde::logMessage($messages_list, __FILE__, __LINE__, PEAR_LOG_ERR);
            $messages_list = array();
        }
        $messages_list = $this->threads->getObjects(array_keys($messages_list), 'DataTreeObject_Message');
        if (empty($messages_list)) {
            return array();
        }

        /* Get a list of forums. */
        require_once AGORA_BASE . '/lib/Forums.php';
        $forums = &Agora_Forums::singleton();
        $forums_list = $forums->getForums(DATATREE_ROOT, false);

        $messages_array = array();
        $url = Horde::applicationUrl('moderate.php');
        $approve_url = Util::addParameter($url, 'approve', true);
        $del_url  = Horde::applicationUrl('messages/delete.php');

        /* Loop through the messages and set up the array. */
        $thread_ids = array_keys($messages_list);
        foreach ($thread_ids as $i => $thread_id) {
            $thread = $messages_list[$thread_id];
            $message_id = $this->threads->getId($thread_id);
            $message = &$this->getMessage($message_id);

            $messages_array[$i] = $message->getMessageArray(false);
            $messages_array[$i]['forum_id'] = Agora::getForumId($thread->name);
            $messages_array[$i]['forum_name'] = $forums_list[$messages_array[$i]['forum_id']];
            $messages_array[$i]['message_name'] = $thread->name;
            $messages_array[$i]['message_author'] = htmlspecialchars($messages_array[$i]['message_author']);
            $messages_array[$i]['message_body'] = Text_Filter::filter($messages_array[$i]['message_body'], 'highlightquotes');
            $messages_array[$i]['message_attachment'] = $message->getAttachmentLink();
            $messages_array[$i]['message_date'] = strftime($GLOBALS['prefs']->getValue('date_format'), $message->data['message_timestamp']);

            if (Auth::isAdmin('agora:admin')) {
                /* TODO: Button to delete the message. */
                //$url = Util::addParameter($del_url, array('id' => $message_id));
                //$messages_array[$i]['actions'][] = Horde::link($url, _("Delete message"), '', '', '', _("Delete message")) . Horde::img('delete.png', _("Delete message"), null, $registry->getImageDir('horde')) . '</a>';
            }

        }

        /* Do the sort. */
        require_once 'Horde/Array.php';
        Horde_Array::arraySort($messages_array, $sort_by, $sort_direction, false);

        /* Set row classes. */
        for ($i = 0; $i < count($messages_array); $i++) {
            if (!isset($messages_array[$i]['class'])) {
                $messages_array[$i]['class'] = 'item' . (($i + 1) % 2);
            }
        }

        return $messages_array;
    }

    /**
     * Approves one or more ids.
     *
     * @access public
     *
     * @param string $action  Whether to 'approve' or 'delete' messages.
     * @param mixed $ids      Either a single message DataTree ID to approve or
     *                        an array of IDs
     *
     * @return mixed  Returns true if successful or otherwise a PEAR_Error.
     */
    function moderate($action, $ids)
    {
        if ($action == 'approve') {
            foreach ($ids as $id) {
                $message = $this->getMessage($id);
                unset($message->data['moderate']);
                $this->threads->updateData($message);
            }
        } elseif ($action == 'delete') {
            foreach ($ids as $id) {
                $this->deleteMessage($id);
            }
        }
    }

    /**
     * Returns the number of threads.
     *
     * @public
     *
     * @param integer $thread_root  Message at which to start the thread counting.
     *
     * @return integer  The number of forums or PEAR_Error on failure.
     */
    function &countThreads($thread_root)
    {
        return $this->threads->count($thread_root);
    }

    /**
     * Returns a table showing the specified message list.
     *
     * @param array $threads      A hash with the thread messages as returned
     *                            by {@link Agora_Messages::getThreads}.
     * @param array $col_headers  A hash with the column headers.
     * @param boolean $bodies     Display the message bodies?
     *
     * @return string  The rendered message table.
     */
    function getThreadsUI($threads, $col_headers, $bodies = false)
    {
        if (!count($threads)) {
            return '';
        }

        /* Render threaded lists with Horde_Tree. */
        if (isset($threads[0]['indent'])) {
            require_once 'Horde/Tree.php';
            $tree = &Horde_Tree::factory('threads', 'html');
            $tree->setOption(array('alternate' => true));
            if ($bodies) {
                $tree->setOption('lines', false);
                $tree->setOption('multiline', true);
            }
            $tree->setHeader(array(
                array('width' => '5%'),
                array('html' => '<b>' . $col_headers['message_thread'] . '</b>',
                      'width' => '50%',
                      'class' => $col_headers['message_thread_class_plain']),
                array('html' => '<b>' . $col_headers['message_author'] . '</b>',
                      'width' => '20%',
                      'class' => $col_headers['message_author_class_plain']),
                array('html' => '<b>' . $col_headers['message_date'] . '</b>',
                      'width' => '25%',
                      'class' => $col_headers['message_date_class_plain'])));
            foreach ($threads as $thread) {
                if ($bodies) {
                    $text = $thread['message_subject'];
                } else {
                    $text = $thread['link'] . $thread['message_subject'] . '</a>';
                }
                if ($bodies) {
                    $text = '<b>' . $text . '</b> ' .
                         $thread['reply'] . '<br />' .
                        str_replace(array("\r", "\n"), '', $thread['message_body'] . $thread['message_attachment']);
                }
                $tree->addNode($thread['message_name'],
                               $thread['parent'],
                               $text,
                               $thread['indent'],
                               true,
                               array('icon' => ''),
                               array($thread['message_author'], $thread['message_date']),
                               array(isset($thread['actions']) ? implode('', $thread['actions']) : ''));
            }
            return $tree->getTree(true);
        }

        /* Set up the thread template tags. */
        $template = &new Horde_Template();
        $template->set('threads_list', $threads);
        $template->set('col_headers', $col_headers);
        $template->set('thread_view_bodies', $bodies, true);

        /* Render template. */
        return $template->fetch(AGORA_TEMPLATES . '/messages/threads.html');
    }

    function _sortByThread($a, $b)
    {
        $a = explode(':', $a['message_name']);
        array_shift($a);
        $b = explode(':', $b['message_name']);
        array_shift($b);

        while (count($a) && count($b)) {
            $a_id = explode('-', array_shift($a));
            $a_id = $a_id[1];
            $b_id = explode('-', array_shift($b));
            $b_id = $b_id[1];
            if ($a_id != $b_id) {
                if ($this->_sortdir) {
                    return $b_id - $a_id;
                } else {
                    return $a_id - $b_id;
                }
            }
        }

        if ($this->_sortdir) {
            return count($b) - count($a);
        } else {
            return count($a) - count($b);
        }
    }

    function getThreadRoot($forum_id, $message_id)
    {
        $name = $this->threads->getName($message_id);
        $parents = $this->threads->getParents($name);
        $root = $current = $name;
        while (is_array($parents) && $current != $forum_id) {
            $root = $current;
            list($current, $parents) = each($parents);
        }

        return $root;
    }

    function setThreadLock($message_id, $lock)
    {
        $thread_root = $this->getThreadRoot($this->_params['forum_id'], $message_id);
        $thread = &$this->threads->getObject($thread_root);
        $thread->data['locked'] = $lock;
        return $this->threads->updateData($thread);
    }

    function isThreadLocked($message_id)
    {
        /* Get the thread root, this is where the lock state is stored. */
        $thread_root = $this->getThreadRoot($this->_params['forum_id'], $message_id);

        /* Get the thread object to examine the data array. */
        $thread = &$this->threads->getObject($thread_root);

        if (isset($thread->data['locked'])) {
            /* If there is a locked state stored return it. */
            return $thread->data['locked'];
        } else {
            /* No stored state, return false. */
            return false;
        }
    }

    function getThreadActions()
    {
        /* Actions. */
        $actions = array();

        $url = Agora::setAgoraId($this->_params['forum_id'], null, Horde::applicationUrl('messages/edit.php'));
        $actions[] = array('url' => $url, 'label' => _("Post message"));

        if (Auth::isAdmin('agora:admin')) {
            $url = Agora::setAgoraId($this->_params['forum_id'], null, Horde::applicationUrl('editforum.php'));
            $actions[] = array('url' => $url, 'label' => _("Edit Forum"));
            $url = Agora::setAgoraId($this->_params['forum_id'], null, Horde::applicationUrl('deleteforum.php'));
            $actions[] = array('url' => $url, 'label' => _("Delete Forum"));
        }

        return $actions;
    }

    function &getForm($vars, $title, $editing = false, $new_forum = false)
    {
        require_once 'Horde/Form.php';
        $form = &Horde_Form::singleton('', $vars, $title);

        /* Set up the form. */
        $form->setButtons(($editing ? _("Save") : _("Post")), true);
        $form->addHidden('', 'url', 'text', false);
        /* Figure out what to do with forum IDs. */
        if ($new_forum) {
            /* This is a new forum to be created, create the var to hold the
             * full path for the new forum. */
            $form->addHidden('', 'new_forum', 'text', false);
        } else {
            /* This is an existing forum so create the forum ID variable. */
            $form->addHidden('', 'forum_id', 'int', false);
        }
        $form->addHidden('', 'message_id', 'int', false);
        $form->addHidden('', 'message_parent_id', 'int', false);
        if (!Auth::getAuth()) {
            $form->addVariable(_("From"), 'posted_by', 'text', true);
        }
        $form->addVariable(_("Subject"), 'message_subject', 'text', true);
        $form->addVariable(_("Message"), 'message_body', 'longtext', true, false, null, array(8, 60, array('emoticons')));

        /* Check if an attachment is available and set variables for deleting
         * and previewing. */
        if ($vars->get('attachment_preview')) {
            $form->addVariable(_("Delete the existing attachment?"), 'attachment_delete', 'boolean', false);
            $form->addVariable(_("Current attachment"), 'attachment_preview', 'html', false);
        }

        if ($this->allowAttachments()) {
            $form->addVariable(_("Attachment"), 'message_attachment', 'file', false);
        }

        return $form;
    }

}

class DataTreeObject_Message extends DataTreeObject {

    var $_messagesOb;

    function DataTreeObject_Message($name)
    {
        parent::DataTreeObject($name);
    }

    function setMessagesOb(&$messagesOb)
    {
        $this->_messagesOb = &$messagesOb;
    }

    function logView()
    {
        global $conf;
        if (!$conf['threads']['track_views']) {
            return;
        }

        $message_id = $this->_messagesOb->getMessageId($this->getName());
        $thread_root = $this->_messagesOb->getThreadRoot($this->_messagesOb->_params['forum_id'], $message_id);
        $thread = &$this->_messagesOb->threads->getObject($thread_root);
        if (!isset($thread->data['views'])) {
            $thread->data['views'] = array();
        }
        $time = time();
        $this->_pruneLog($thread->data['views'], $time);

        if (!isset($thread->data['views'][$time])) {
            $thread->data['views'][$time] = 1;
        } else {
            $thread->data['views'][time()]++;
        }
        return $this->_messagesOb->threads->updateData($thread);
    }

    function isHot()
    {
        global $conf, $prefs;
        if (!$conf['threads']['track_views']) {
            return;
        }

        $time = time();
        $message_id = $this->_messagesOb->getMessageId($this->getName());
        $thread_root = $this->_messagesOb->getThreadRoot($this->_messagesOb->_params['forum_id'], $message_id);
        $thread = &$this->_messagesOb->threads->getObject($thread_root);
        if (!isset($thread->data['views'])) {
            return false;
        }
        $this->_pruneLog($thread->data['views'], $time);

        return (count($thread->data['views']) > $prefs->getValue('threads_hot'));
    }

    function _pruneLog(&$log, $time)
    {
        $cutoff = $time - 86400;
        /* Make sure the log has only the last 24hrs. Would be nice to
         * have a better way to split off only the views relevant for
         * the last 24hrs other than foreaching through everything. */
        foreach ($log as $log_time => $views) {
            if ($log_time < $cutoff) {
                unset($log[$log_time]);
            }
        }
    }

    function isLocked()
    {
        $message_id = $this->_messagesOb->getMessageId($this->getName());
        return $this->_messagesOb->isThreadLocked($message_id);
    }

    /**
     * Checks if a message needs moderating by looking at whether the
     * "moderate" flag has been set.
     *
     * @access public
     *
     * @return bool
     */
    function needsModerating()
    {
        return !empty($this->data['moderate']);
    }

    /**
     * Returns the message object as an array, for example for use in forms or
     * lists.
     *
     * @access public
     */
    function getMessageArray($check_lock = true)
    {
        if (isset($this->_messagesOb->_params['forum_id'])) {
            $array['forum_id'] = $this->_messagesOb->_params['forum_id'];
        }
        $array['message_id'] = $this->_messagesOb->getMessageId($this->getName());

        $array['plain_subject'] = $this->data['message_subject'];
        $array['plain_body'] = $this->data['message_body'];

        require_once 'Horde/Text/Filter.php';
        $array['message_subject'] = htmlspecialchars($this->data['message_subject']);
        $filters = array('text2html');
        $filter_params = array(array('parselevel' => TEXT_HTML_MICRO));

        /* Convert any emoticons in body and subject to graphics if
         * required. */
        if ($GLOBALS['prefs']->getValue('message_emoticons')) {
            $array['message_subject'] = Text_Filter::filter($array['message_subject'], 'emoticons', array(true));
            $filters[] = 'emoticons';
            $filter_params[] = array(true);
        }

        $array['message_body'] = Text_Filter::filter($this->data['message_body'], $filters, $filter_params);

        $array['message_author'] = $this->data['message_author'];
        if ($check_lock) {
            $array['message_lock'] = $this->isLocked();
        }
        $array['message_timestamp'] = $this->data['message_timestamp'];
        if (!empty($this->data['message_attachment'])) {
            $array['message_attachment'] = $this->data['message_attachment'];
        }

        return $array;
    }

    function getAttachmentLink()
    {
        if (empty($this->data['message_attachment'])) {
            return '';
        }

        global $mime_drivers, $mime_drivers_map;
        require_once 'Horde/MIME/Part.php';
        require_once 'Horde/MIME/Viewer.php';
        require_once 'Horde/MIME/Magic.php';
        require_once 'Horde/MIME/Contents.php';

        /* Set up the view url. */
        $url = Horde::applicationUrl('view.php');
        $params = array('action_id'  => 'download',
                        'file_type'  => $this->data['message_attachment']['type'],
                        'forum_id'   => $this->_messagesOb->_params['forum_id'],
                        'message_id' => $this->_messagesOb->getMessageId($this->getName()),

                        'file_name'  => $this->data['message_attachment']['name']);
        $url = Util::addParameter($url, $params);

        /* Make sure we have the tooltips javascript. */
        Horde::addScriptFile('tooltip.js', 'horde', true);

        /* Set up the icon and the title. */
        if (!empty($this->data['message_attachment']['type'])) {
            $mime_icon = MIME_Viewer::getIcon($this->data['message_attachment']['type']);
        } else {
            $mime_icon = MIME_Viewer::getIcon(MIME_Magic::filenameToMIME($this->data['message_attachment']['name']));
        }
        $title = _("download") . ': ' . $this->data['message_attachment']['name'];
        $tooltip = $title . "\n" . sprintf(_("size: %s"), $this->data['message_attachment']['size']) . "\n" . sprintf(_("type: %s"), $this->data['message_attachment']['type']);

        /* Return the link with a tooltip for further info on the download. */
        return Horde::linkTooltip($url, $title, '', '', '', $tooltip) . Horde::img($mime_icon, $title, 'align="middle"', '') . '&nbsp;' . $this->data['message_attachment']['name'] . '</a>&nbsp;&nbsp;';
    }

    function _toAttributes()
    {
        $attributes[] = array('name'  => 'message',
                              'key'   => 'forum_id',
                              'value' => $this->data['forum_id']);
        $attributes[] = array('name'  => 'message',
                              'key'   => 'subject',
                              'value' => $this->data['message_subject']);
        $attributes[] = array('name'  => 'message',
                              'key'   => 'body',
                              'value' => $this->data['message_body']);
        $attributes[] = array('name'  => 'message',
                              'key'   => 'author',
                              'value' => $this->data['message_author']);
        $attributes[] = array('name'  => 'message',
                              'key'   => 'timestamp',
                              'value' => $this->data['message_timestamp']);
        if (isset($this->data['message_attachment'])) {
            foreach ($this->data['message_attachment'] as $key => $val) {
                $attributes[] = array('name'  => 'attachment',
                                      'key'   => $key,
                                      'value' => $val);
            }
        }
        if (isset($this->data['moderate'])) {
            $attributes[] = array('name'  => 'message',
                                  'key'   => 'moderate',
                                  'value' => true);
        }
        return $attributes;
    }

    function _fromAttributes($attributes)
    {
        /* Initialise the data array. */
        $this->data = array();

        foreach ($attributes as $attr) {
            if ($attr['key'] == 'forum_id') {
                $this->data['forum_id'] = $attr['value'];
            } elseif ($attr['name'] == 'attachment') {
                /* The message has an attachment. */
                $this->data['message_attachment'][$attr['key']] = $attr['value'];
            } elseif ($attr['key'] == 'moderate') {
                /* The message still needs to be moderated. */
                $this->data['moderate'] = true;
            } else {
                $this->data[$attr['name'] . '_' . $attr['key']] = $attr['value'];
            }
        }
    }

}
