<?php
/**
 * Agora external API interface.
 *
 * This file defines Agora's external API interface. Other
 * applications can interact with Agora through this API.
 *
 * $Horde: agora/lib/api.php,v 1.45 2005/01/03 14:34:46 jan Exp $
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * See the enclosed file COPYING for license information (GPL). If you did not
 * receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Marko Djukic <marko@oblo.com>
 * @version $Revision: 1.45 $
 * @package Agora
 */

$_services['perms'] = array(
    'args' => array(),
    'type' => 'stringArray'
);

$_services['list'] = array(
    'args' => array('forum_id' => 'int'),
    'type' => '{urn:horde}stringArray'
);

$_services['saveForum'] = array(
    'args' => array('app' => 'string',
                    'parent' => 'string',
                    'info' => '{urn:horde}hash',
                    'auto_create' => 'bool'),
    'type' => 'int'
);

$_services['getMessages'] = array(
    'args' => array('forum_name' => 'string',
                    'order' => 'string',
                    'bodies' => 'boolean',
                    'app' => 'string',
                    'replay_url' => 'string'),
    'type' => '{urn:horde}hash'
);

$_services['getThreads'] = array(
    'args' => array('forum_name' => 'string',
                    'bodies' => 'boolean',
                    'app' => 'string',
                    'replay_url' => 'string'),
    'type' => '{urn:horde}hash'
);

$_services['numMessages'] = array(
    'args' => array('forum_name' => 'string',
                    'app' => 'string'),
    'type' => 'int'
);

$_services['renderThreads'] = array(
    'args' => array('forum_name' => 'string',
                    'bodies' => 'boolean',
                    'app' => 'string',
                    'replay_url' => 'string'),
    'type' => 'string'
);

$_services['postMessage'] = array(
    'args' => array('app' => 'string',
                    'forum_name' => 'string',
                    'callback' => 'string',
                    'params' => '{urn:horde}hash',
                    'url' => 'string'),
    'type' => 'string'
);


function _agora_perms()
{
    $perms['tree']['agora']['admin'] = true;
    $perms['title']['agora:admin'] = _("Admin");

    return $perms;
}

/**
 * Get back a list of available forums.
 *
 * @param integer $forum_id  Supplying this parameter will return a list of
 *                           child forums of the requested forum id.
 */
function _agora_list($forum_id = DATATREE_ROOT)
{
    @define('AGORA_BASE', dirname(__FILE__) . '/..');
    require_once AGORA_BASE . '/lib/base.php';
    require_once AGORA_BASE . '/lib/Forums.php';

    $forums = &Agora_Forums::singleton();
    return $forums->getForums($forum_id, true, 'forum_name', 0, true);
}

/**
 * Create or modify an agora forum. This is used for apps to create
 * forums for their own use. They will not show up in the regular
 * agora forum view since they will be using a datatree group
 * 'agora.forums.<appname>'.
 *
 * @param string $app     The Horde application that is saving this forum.
 * @param string $parent  The parent forum.
 * @param array $info     The forum information to save consisting of:
 *                          forum_parent_id
 *                          forum_name
 *                          forum_description
 *                          forum_moderated
 *                          forum_attachments
 */
function _agora_saveForum($app, $parent, $info, $auto_create = true)
{
    @define('AGORA_BASE', dirname(__FILE__) . '/..');
    require_once AGORA_BASE . '/lib/base.php';
    require_once AGORA_BASE . '/lib/Forums.php';

    $forums = &Agora_Forums::singleton('agora.forums.' . $app);

    $info['forum_parent_id'] = (empty($parent)) ? DATATREE_ROOT : $forums->getForumId($parent);

    return $forums->saveForum($info);
}

/**
 * Returns all messages of a forum.
 *
 * @param string $forum_name  The unique name for the forum.
 * @param string $order       Return messages sorted by this property.
 * @param boolean $bodies     Whether to include message bodies in the view.
 * @param string $app         The application that the specified forum belongs
 *                            to.
 * @param string $reply_url   An alternative URL where the reply link points
 *                            to. Takes "%p" as a placeholder for parent
 *                            message ID.
 *
 * @return array  All messages of the specified forum.
 */
function &_agora_getMessages($forum_name, $order = 'message_timestamp', $bodies = false, $app = '', $reply_url = null)
{
    @define('AGORA_BASE', dirname(__FILE__) . '/..');
    require_once AGORA_BASE . '/lib/base.php';
    require_once AGORA_BASE . '/lib/Forums.php';
    require_once AGORA_BASE . '/lib/Messages.php';

    if (!empty($app)) {
        $app = '.' . $app;
    }
    $forums = &Agora_Forums::singleton('agora.forums' . $app);

    if (!$forums->exists($forum_name)) {
        return array();
    }
    if (is_a($forum_id = $forums->getForumId($forum_name), 'PEAR_Error')) {
        return $forum_id;
    }

    $messages = &Agora_Messages::singleton(array('data_root' => 'agora.forums' . $app, 'forum_id' => $forum_id));
    return $messages->getThreads($forum_id, true, $order, 0, false, '', $reply_url);
}

/**
 * Returns all messages of a forum, in a threaded order.
 *
 * @param string $forum_name  The unique name for the forum.
 * @param boolean $bodies     Whether to include message bodies in the view.
 * @param string $app         The application that the specified forum belongs
 *                            to.
 * @param string $reply_url   An alternative URL where the reply link points
 *                            to. Takes "%p" as a placeholder for parent
 *                            message ID.
 *
 * @return array  All messages of the specified forum.
 */
function &_agora_getThreads($forum_name, $bodies = false, $app = '', $reply_url = null)
{
    return _agora_getMessages($forum_name, 'message_thread', $bodies, $app, $reply_url);
}

/**
 * Returns the number of messages in a forum,
 *
 * @param string $forum_name  The unique name for the forum.
 * @param string $app         The application that the specified forum belongs
 *                            to.
 *
 * @return int  The number of messages in the specified forum.
 */
function _agora_numMessages($forum_name, $app = '')
{
    return count(_agora_getThreads($forum_name, false, $app));
}

/**
 * Returns all threads of a forum in a threaded view.
 *
 * @param string $forum_name  The unique name for the forum.
 * @param boolean $bodies     Whether to include message bodies in the view.
 * @param string $app         The application that the specified forum belongs
 *                            to.
 * @param string $reply_url   An alternative URL where the reply link points
 *                            to. Takes "%p" as a placeholder for parent
 *                            message ID.
 *
 * @return string  The HTML code of the thread view.
 */
function _agora_renderThreads($forum_name, $bodies = false, $app = '', $reply_url = null)
{
    $threads = &_agora_getThreads($forum_name, $bodies, $app, $reply_url);
    if (!count($threads)) {
        return '';
    }

    $col_headers = array('message_thread' => _("Subject"),
                         'message_thread_class_plain' => '',
                         'message_author' => _("Posted By"),
                         'message_author_class_plain' => '',
                         'message_date' => _("Date"),
                         'message_date_class_plain' => '');

    if (!empty($app)) {
        $app = '.' . $app;
    }
    $forums = &Agora_Forums::singleton('agora.forums' . $app);
    $forum_id = $forums->getForumId($forum_name);
    $messages = &Agora_Messages::singleton(array('data_root' => 'agora.forums' . $app, 'forum_id' => $forum_id));
    return '<table border="0" cellpadding="2" cellspacing="0" width="100%">' .
        '<tr><td align="left" class="header"><b>' .
        _("Comments") .
        '</b></td></tr></table>' .
        $messages->getThreadsUI($threads, $col_headers, $bodies);
}

/**
 * Allows other Horde apps to post messages.
 *
 * The forum name is constructed by just the $forum_name variable
 * under the data root 'agora.forums.<app>'. It is up to the apps
 * themselves to make sure that the forum name is unique.
 *
 * If the forum does not exist, it will be automatically created by
 * Agora.
 *
 * @access private
 *
 * @param string $app         The application which is posting this message.
 * @param string $forum_name  The unique name for the forum.
 * @param string $callback    A callback method of the specified application
 *                            that gets called to make sure that posting to
 *                            this forum is allowed.
 * @param array $params       Any parameters for the forum message posting.
 * <pre>
 * message_id        - An existing message to edit
 * message_parent_id - The ID of the parent message
 * title             - Posting title
 * </pre>
 * @param string $url         If specified, the form gets submitted to this URL
 *                            instead of the current page.
 * @param array $variables    A hash with all variables of a submitted form
 *                            generated by this method.
 *
 * @return mixed  Returns either the rendered Horde_Form for posting a message
 *                or PEAR_Error object on error, or true in case of a
 *                successful post.
 */
function _agora_postMessage($app, $forum_name, $callback, $params = array(),
                            $url = null, $variables = null)
{
    global $registry;

    /* Check if posting messages is allowed. */
    if (is_a($check = $registry->callByPackage($app, $callback, array($forum_name)), 'PEAR_Error') ||
        !$check) {
        return '';
    }

    @define('AGORA_BASE', dirname(__FILE__) . '/..');
    require_once AGORA_BASE . '/lib/base.php';
    require_once AGORA_BASE . '/lib/Forums.php';
    require_once AGORA_BASE . '/lib/Messages.php';
    require_once 'Horde/Form.php';
    require_once 'Horde/Variables.php';

    /* Create a separate notification queue. */
    $queue = &new Notification('agoraPostMessage');
    $queue->attach('status');

    /* Set up the forums object. */
    $forums = &Agora_Forums::singleton('agora.forums.' . $app);

    /* Set up form variables. */
    $vars = &new Variables($variables);
    $formname = $vars->get('formname');

    /* Check if the forum exists and fetch the ID. */
    if ($forums->exists($forum_name)) {
        $params['forum_id'] = $forums->getForumId($forum_name);
        $vars->set('forum_id', $params['forum_id']);
    } else {
        $vars->set('new_forum', $forum_name);
        $params['forum_id'] = null;
    }

    if (!isset($params['message_id'])) {
        $params['message_id'] = null;
        $editing = false;
    } else {
        $editing = true;
    }

    /* Set up the messages control object. */
    $m_params = array('message_id' => $params['message_id'],
                      'forum_id'   => $params['forum_id'],
                      'data_root'  => 'agora.forums.' . $app);
    $messages = &Agora_Messages::singleton($m_params);
    if (is_a($messages, 'PEAR_Error')) {
        $queue->push(sprintf(_("Could not post the message: %s"), $messages->getMessage), 'horde.error');
        return Util::bufferOutput(array($queue, 'notify'), array('listeners' => 'status'));
    }

    /* Set a default title if one not specified. */
    if (!isset($params['title'])) {
        $params['title'] = ($editing) ? _("Edit Message") : _("Post a New Message");
    }

    /* Get the form object. */
    $form = $messages->getForm($vars, $params['title'], $editing, is_null($params['forum_id']));

    /* Validate the form. */
    if ($form->validate($vars)) {
        $form->getInfo($vars, $info);

        if (isset($info['new_forum'])) {
            /* TODO: For now set the additional forum data to some
             * defaults.  Some way of setting them would need to be
             * figured out.*/
            $forum_info = array('forum_name' => $info['new_forum'],
                                'forum_parent_id' => DATATREE_ROOT,
                                'forum_moderated' => false,
                                'forum_description' => '',
                                'forum_attachments' => false);
            $info['forum_id'] = $m_params['forum_id'] = $forums->saveForum($forum_info);
            $success = Agora_Messages::setupForumObject($messages, $m_params);
        }

        /* Try and store this message and get back a new message_id */
        $message_id = $messages->saveMessage($info);
        if (is_a($message_id, 'PEAR_Error')) {
            $queue->push(sprintf(_("Could not post the message. %s"), $message_id->getMessage()), 'horde.error');
        } else {
            $queue->push(_("Message posted."), 'horde.success');
            return Util::bufferOutput(array($queue, 'notify'), array('listeners' => 'status'));
        }
    }

    /* Replying to a previous post? */
    if (isset($params['message_parent_id']) &&
        !$form->isSubmitted()) {
        $message_array = $messages->replyMessage($params['message_parent_id']);
        if (!is_a($message_array, 'PEAR_Error')) {
            $vars->set('message_parent_id', $params['message_parent_id']);
            $vars->set('message_subject', $message_array['plain_subject']);
            $vars->set('message_body', $message_array['plain_body']);
        } else {
            /* Bad parent message id, offer to do a regular post. */
            $vars->set('message_parent_id', '');
        }
    }

    /* Render the form. */
    require_once 'Horde/Form/Renderer.php';
    $renderer = &new Horde_Form_Renderer();
    if (!$url) {
        $url = Horde::selfUrl(true);
    }
    return Util::bufferOutput(array($queue, 'notify'), array('listeners' => 'status')) .
        Util::bufferOutput(array($form, 'renderActive'), $renderer, $vars, $url, 'post', null, false);
}
