<?php
/**
 * The Agora script to delete a message.
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * See the enclosed file COPYING for license information (GPL). If you did not
 * receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * $Horde: agora/messages/delete.php,v 1.31 2005/01/03 14:34:47 jan Exp $
 */

@define('AGORA_BASE', dirname(__FILE__) . '/..');
require_once AGORA_BASE . '/lib/base.php';
require_once AGORA_BASE . '/lib/Messages.php';
require_once 'Horde/Form.php';
require_once 'Horde/Variables.php';

/* Only admin should be using this. */
if (!Auth::isAdmin('agora:admin')) {
    Horde::authenticationFailureRedirect();
}

$vars = &Variables::getDefaultVariables();
$formname = $vars->get('formname');
list($forum_id, $message_id) = Agora::getAgoraId();
if (empty($message_id)) {
    header('Location: ' . Horde::applicationUrl('forums.php', true));
    exit;
}

/* Set up the messages control object. */
$params = array('message_id' => $message_id,
                'forum_id' => $forum_id);
$messages = &Agora_Messages::singleton($params);

/* Get the message to be deleted. */
if ($message_id && !$formname) {
    $message = &$messages->getMessage($message_id);
    $vars = &new Variables($message->getMessageArray());
}

/* Check if valid doing this delete message. */
if (is_a($messages, 'PEAR_Error')) {
    $notification->push(sprintf(_("Could not delete the message. %s"), $messages->getMessage()), 'horde.error');
    header('Location: ' . Horde::applicationUrl('forums.php', true));
    exit;
}

/* Get the forum details. */
$forum_name = $messages->forum->getShortName();

/* Get message information. */
$message = $messages->getMessage($message_id);
$message_array = $message->getMessageArray();

/* Get the form object. */
$form = &Horde_Form::singleton('', $vars, sprintf(_("Delete '%s' and all replies?"), $message_array['message_subject']));

/* Set up the form. */
$form->setButtons(array(_("Delete"), _("Cancel")));
$form->addHidden('', 'forum_id', 'int', false);
$form->addHidden('', 'message_id', 'int', false);

if ($vars->get('submitbutton') == _("Delete")) {
    /* Check if the form is valid and process. */
    $form->validate($vars);

    if ($form->isValid()) {
        $form->getInfo($vars, $info);

        /* Try and delete this message. */
        $result = $messages->deleteMessage($message_id);
        if (is_a($result, 'PEAR_Error')) {
            $notification->push(sprintf(_("Could not delete the message. %s"), $result->getMessage()), 'horde.error');
        } else {
            $notification->push(_("Message deleted."), 'horde.success');
            $url = Agora::setAgoraId($forum_id, null, Horde::applicationUrl('threads.php', true));
            header('Location: ' . $url);
            exit;
        }
    }
} elseif ($vars->get('submitbutton') == _("Cancel")) {
    $notification->push(_("Message not deleted."), 'horde.message');
    $url = Agora::setAgoraId($forum_id, $message_id, Horde::applicationUrl('messages/index.php', true));
    header('Location: ' . $url);
    exit;
}

/* Set up template data. */
$template = &new Horde_Template();
$template->set('message_subject', $message_array['message_subject']);
$template->set('message_author', $message_array['message_author']);
$template->set('message_date', strftime($prefs->getValue('date_format'), $message_array['message_timestamp']));
$template->set('message_body', $message_array['message_body']);
$template->set('menu', Agora::getMenu('string'));
$template->set('notify', Util::bufferOutput(array($notification, 'notify'), array('listeners' => 'status')));

/* Render the form. */
require_once 'Horde/Form/Renderer.php';
$renderer = &new Horde_Form_Renderer();
$template->set('formbox', Util::bufferOutput(array($form, 'renderActive'), $renderer, $vars, 'delete.php', 'post'));

require AGORA_TEMPLATES . '/common-header.inc';
echo $template->fetch(AGORA_TEMPLATES . '/messages/form.html');
require $registry->get('templates', 'horde') . '/common-footer.inc';
