<?php
/**
 * The Agora script to post a new message, edit an existing message,
 * or reply to a message.
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * See the enclosed file COPYING for license information (GPL). If you did not
 * receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * $Horde: agora/messages/edit.php,v 1.54 2005/01/03 14:34:47 jan Exp $
 */

@define('AGORA_BASE', dirname(__FILE__) . '/..');
require_once AGORA_BASE . '/lib/base.php';
require_once AGORA_BASE . '/lib/Messages.php';
require_once 'Horde/Form.php';
require_once 'Horde/Variables.php';

$vars = &Variables::getDefaultVariables();
$formname = $vars->get('formname');
list($forum_id, $message_id) = Agora::getAgoraId();
$message_parent_id = Util::getFormData('message_parent_id');

/* Set up the messages control object. */
$params = array('message_id' => $message_id,
                'forum_id'   => $forum_id);
$messages = &Agora_Messages::singleton($params);
if (is_a($messages, 'PEAR_Error')) {
    $notification->push(sprintf(_("Could not post the message: %s"), $messages->getMessage), 'horde.warning');
    $url = Horde::applicationUrl('forums.php', true);
    header('Location: ' . $url);
    exit;
}

/* Check if a message is being edited. */
if ($message_id) {
    if (!Auth::getAuth()) {
        header('Location: ' . Horde::applicationUrl('forums.php', true));
        exit;
    }

    $message = &$messages->getMessage($message_id);
    if (!$formname) {
        $message_array = $message->getMessageArray();
        $vars = &new Variables($message_array);
        $vars->set('message_subject', $message_array['plain_subject']);
        $vars->set('message_body', $message_array['plain_body']);
    }
    $attachment_link = $message->getAttachmentLink();
    if ($attachment_link) {
        $vars->set('attachment_preview', $attachment_link);
    }
} else {
    $vars->set('forum_id', $forum_id);
    $vars->set('message_id', $message_id);
}

/* Get the forum details. */
$forum_name = $messages->forum->getShortName();

/* Set the title. */
$title = $message_parent_id ?
    sprintf(_("Post a Reply to \"%s\""), $forum_name) :
    ($message_id ? sprintf(_("Edit Message in \"%s\""), $forum_name) :
                   sprintf(_("Post a New Message to \"%s\""), $forum_name));

/* Get the form object. */
$form = $messages->getForm($vars, $title, $message_id);

/* Validate the form. */
if ($form->validate($vars)) {
    $form->getInfo($vars, $info);

    /* Try and store this message and get back a new message_id */
    $message_id = $messages->saveMessage($info);
    if (is_a($message_id, 'PEAR_Error')) {
        $notification->push(sprintf(_("Could not post the message. %s"), $message_id->getMessage()), 'horde.error');
    } else {
        $notification->push(_("Message posted."), 'horde.success');
        if (!empty($info['url'])) {
            $url = Horde::url($info['url'], true);
        } else {
            $url = Agora::setAgoraId($forum_id, $message_id, Horde::applicationUrl('messages/index.php', true));
        }
        header('Location: ' . $url);
        exit;
    }
}

/* Template object. */
$template = &new Horde_Template();

/* Check if a parent message exists and set up tags accordingly. */
if ($message_parent_id) {
    $message_array = $messages->replyMessage($message_parent_id);
    if (!is_a($message_array, 'PEAR_Error')) {
        $vars->set('message_subject', $message_array['plain_subject']);
        $vars->set('message_body', $message_array['plain_body']);
        $template->set('message_subject', $message_array['message_subject']);
        $template->set('message_from', $message_array['message_from']);
        $template->set('message_body', $message_array['message_body']);
    } else {
        /* Bad parent message id, offer to do a regular post. */
        $message_parent_id = null;
        $vars->set('message_parent_id', '');
        $notification->push(_("Invalid parent message, you will be posting this message as a new thread."), 'horde.warning');
    }
}

$template->set('replying', $message_parent_id, true);
$template->set('menu', Agora::getMenu('string'));
$template->set('notify', Util::bufferOutput(array($notification, 'notify'), array('listeners' => 'status')));

/* Render the form. */
require_once 'Horde/Form/Renderer.php';
$renderer = &new Horde_Form_Renderer();
$template->set('formbox', Util::bufferOutput(array($form, 'renderActive'), $renderer, $vars, 'edit.php', 'post'));

require AGORA_TEMPLATES . '/common-header.inc';
echo $template->fetch(AGORA_TEMPLATES . '/messages/edit.html');
require $registry->get('templates', 'horde') . '/common-footer.inc';
