<?php
/**
 * The Agora script to display a forum post.
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * $Horde: agora/messages/message.php,v 1.66 2005/01/03 14:34:47 jan Exp $
 */

@define('AGORA_BASE', dirname(__FILE__) . '/..');
require_once AGORA_BASE . '/lib/base.php';
require_once AGORA_BASE . '/lib/Messages.php';
require_once 'Horde/Text.php';

list($forum_id, $message_id) = Agora::getAgoraId();
$params = array('message_id' => $message_id,
                'forum_id' => $forum_id);

/* Set up the message object. */
$messages = &Agora_Messages::singleton($params);

/* Get requested message, if fail then back to forums list. */
$message = $messages->getMessage($message_id);
if (is_a($message, 'PEAR_Error')) {
    $notification->push(sprintf(_("Could not open the message. %s"), $message->getMessage()), 'horde.warning');
    header('Location: ' . Horde::applicationUrl('forums.php', true));
    exit;
} elseif (empty($message)) {
    $notification->push(_("Message does not exist."), 'horde.warning');
    header('Location: ' . Horde::applicationUrl('forums.php', true));
    exit;
}

/* Log views if required. */
$message->logView();

/* Get the sorting. */
$sort_by = Agora::getSortBy('thread');
$sort_dir = Agora::getSortDir('thread');

/* Get the message array and the sorted thread list. */
$message_array = $message->getMessageArray();
$threads_list = $messages->getThreads($messages->getThreadRoot($forum_id, $message_id),
                                      true, $sort_by, $sort_dir, true);

/* Set up the column headers. */
$col_headers = array(array('message_thread' => _("Thread"), 'message_subject' => _("Subject")), 'message_author' => _("Posted By"), 'message_date' => _("Date"));
$col_headers = Agora::formatColumnHeaders($col_headers, $sort_by, $sort_dir, 'thread');

/* Set up the template tags. */
$threads = $messages->getThreadsUI($threads_list, $col_headers);
$template = &new Horde_Template();
$template->set('message_id', $message_array['message_id']);
$template->set('message_from', sprintf(_("Posted by %s on %s"), htmlspecialchars($message_array['message_author']), strftime($prefs->getValue('date_format'), $message_array['message_timestamp'])));
$template->set('message_subject', $message_array['message_subject']);
$template->set('message_body', Text_Filter::filter($message_array['message_body'], 'highlightquotes'));
$template->set('message_attachment', $message->getAttachmentLink(), true);
$url = Agora::setAgoraId($forum_id, $message_id, Horde::applicationUrl('messages/index.php'));
$template->set('menu', Agora::getMenu('string'));
$template->set('notify', Util::bufferOutput(array($notification, 'notify'), array('listeners' => 'status')));

/* Actions. */
$actions = array();

/* Check if the thread allows replies. */
if (!$message->isLocked()) {
    $url = Agora::setAgoraId($forum_id, null, Horde::applicationUrl('messages/edit.php'));
    $url = Util::addParameter($url, 'message_parent_id', $message_id);
    $actions[] = Horde::link($url, _("Reply")) . _("Reply") . '</a>';
}

if (Auth::isAdmin('agora:admin')) {
    $url = Agora::setAgoraId($forum_id, $message_id, Horde::applicationUrl('messages/edit.php'));
    $actions[] = Horde::link($url, _("Edit")) . _("Edit") . '</a>';

    $url = Agora::setAgoraId($forum_id, $message_id, Horde::applicationUrl('messages/delete.php'));
    $actions[] = Horde::link($url, _("Delete")) . _("Delete") . '</a>';

    $url = Agora::setAgoraId($forum_id, $message_id, Horde::applicationUrl('messages/lock.php'));
    $label = ($message->isLocked()) ? _("Unlock thread") : _("Lock thread");
    $actions[] = Horde::link($url, $label) . $label . '</a>';
}

$template->setOption('gettext', true);
$template->set('actions', $actions, true);
$template->set('threads', $threads, true);

$title = $message_array['plain_subject'];
require AGORA_TEMPLATES . '/common-header.inc';
echo $template->fetch(AGORA_TEMPLATES . '/messages/message.html');
require $registry->get('templates', 'horde') . '/common-footer.inc';
