<?php
/**
 * The Agora script to moderate any outstanding messages requiring moderation.
 *
 * Copyright 2003-2005 The Horde Project (http://www.horde.org/)
 *
 * $Horde: agora/moderate.php,v 1.5 2005/01/03 14:34:46 jan Exp $
 *
 * See the enclosed file COPYING for license information (GPL). If you did not
 * receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

@define('AGORA_BASE', dirname(__FILE__));
require_once AGORA_BASE . '/lib/base.php';
require_once AGORA_BASE . '/lib/Messages.php';
require_once 'Horde/Text.php';
require_once 'Horde/Variables.php';
require_once 'Horde/UI/Pager.php';

/* Only admin should be using this. */
if (!Auth::isAdmin('agora:admin')) {
    Horde::authenticationFailureRedirect();
}

/* Set up the messages object. */
$messages = &Agora_Messages::singleton();

/* Check for any actions. */
if ($action = Util::getFormData('action')) {
    $message_ids = Util::getFormData('message_ids');
    $messages->moderate($action, $message_ids);
    header('Location: ' . Horde::applicationUrl('moderate.php', true));
    exit;
}
/* Which page are we on?  Default to page 0. */
$messages_page = Util::getFormData('page', 0);
$messages_per_page = $prefs->getValue('threads_per_page');
$messages_start = $messages_page * $messages_per_page;

/* Get the sorting. */
$sort_by = Agora::getSortBy('moderate');
$sort_dir = Agora::getSortDir('moderate');

/* Get a list of messages still to moderate. */
$messages_list = $messages->getModerateList($sort_by, $sort_dir);
if (empty($messages_list)) {
    $messages_count = 0;
    $notification->push(_("No messages to moderate available."), 'horde.success');
} else {
    $messages_count = count($messages_list);
    $messages_list = array_slice($messages_list, $messages_start, $messages_per_page);
}

/* Set up the column headers. */
$col_headers = array('forum_name' => _("Forum"), 'message_subject' => _("Subject"), 'message_author' => _("Posted By"), 'message_body' => _("Body"), 'message_date' => _("Date"));
$col_headers = Agora::formatColumnHeaders($col_headers, $sort_by, $sort_dir, 'moderate');

/* JS for moderating. */
Horde::addScriptFile('moderate.js');

/* Set up actions. */
$url = Horde::applicationUrl('moderate.php');
$actions = array();
$actions[] = array('js' => "moderate('approve'); return false;", 'label' => _("Approve"));
$actions[] = array('js' => "moderate('delete'); return false;", 'label' => _("Delete"));

/* Set up the template tags. */
$template = &new Horde_Template();
$template->set('col_headers', $col_headers);
$template->set('messages', $messages_list, true);
$template->set('actions', $actions);
$template->set('menu', Agora::getMenu('string'));
$template->set('notify', Util::bufferOutput(array($notification, 'notify'), array('listeners' => 'status')));

/* Set the gettext option. */
$template->setOption('gettext', true);

/* Set up pager. */
$vars = &Variables::getDefaultVariables();
$pager_ob = &new Horde_UI_Pager('moderate_page', $vars, array('num' => $messages_count, 'url' => 'moderate.php', 'perpage' => $messages_per_page));
$pager_ob->preserve('agora', Util::getFormData('agora'));
$template->set('pager_link', $pager_ob->render(), true);

$title = _("Messages Awaiting Moderation");
require AGORA_TEMPLATES . '/common-header.inc';
echo $template->fetch(AGORA_TEMPLATES . '/moderate/moderate.html');
require $registry->get('templates', 'horde') . '/common-footer.inc';
