#!/usr/local/bin/php
<?php
/**
 * $Horde: agora/scripts/upgrades/2004-04-07_migrate_datatree.php,v 1.1 2004/09/18 17:42:54 mdjukic Exp $
 *
 * Script to migrate the old way of storing forum info in horde_datatree data
 * field to the new horde_datatree_attributes table.
 * Before running this script you MUST run the category->datatree migration
 * script found in: horde/scripts/rename_category.php
 */

/**
 * Set this to true if you want DB inserts done.
 */
$live = false;

// No need for auth.
@define('AUTH_HANDLER', true);

// Find the base file paths.
@define('HORDE_BASE', dirname(__FILE__) . '/../../..');
@define('AGORA_BASE', dirname(__FILE__) . '/../..');

// Do CLI checks and environment setup first.
require_once HORDE_BASE . '/lib/core.php';
require_once 'Horde/CLI.php';

// Make sure no one runs this from the web.
if (!Horde_CLI::runningFromCLI()) {
    exit("Must be run from the command line\n");
}

// Load the CLI environment - make sure there's no time limit, init
// some variables, etc.
Horde_CLI::init();

require_once 'DB.php';
require_once HORDE_BASE . '/lib/base.php';
require_once 'Horde/Serialize.php';
require_once 'Horde/Array.php';

/* Open the database. */
$db = &DB::connect($conf['sql']);
if (is_a($db, 'PEAR_Error')) {
    var_dump($db); exit;
}


/* Get the available forums. */
$forums = $db->getAll('SELECT * FROM horde_datatree WHERE group_uid = \'agora.forums\'', DB_FETCHMODE_ASSOC);
$attr = array();
foreach ($forums as $row) {
    $data = Horde_Serialize::unserialize($row['datatree_data'], $row['datatree_serialized'], NLS::getCharset());
    if (isset($data['message_seq'])) {
        $attr[] = array('id'    => $row['datatree_id'],
                        'name'  => 'message',
                        'key'   => 'seq',
                        'value' => $data['message_seq']);
    }
    if (isset($data['forum_moderated'])) {
        $attr[] = array('id'    => $row['datatree_id'],
                        'name'  => 'forum',
                        'key'   => 'moderated',
                        'value' => $data['forum_moderated']);
    }
    if (isset($data['forum_description'])) {
        $attr[] = array('id'    => $row['datatree_id'],
                        'name'  => 'forum',
                        'key'   => 'description',
                        'value' => $data['forum_description']);
    }
}

/* Do the inserts if live. */
if ($live) {
    foreach ($attr as $a) {
        $sql = sprintf('INSERT INTO horde_datatree_attributes (datatree_id, attribute_name, attribute_key, attribute_value) VALUES(%s, %s, %s, %s)',
                       (int)$a['id'],
                       $db->quote($a['name']),
                       $db->quote($a['key']),
                       $db->quote($a['value']));
        $result = $db->query($sql);
        if (is_a($result, 'PEAR_Error')) {
            var_dump($result);
            exit;
        }
        /* Reset the datatree_data field to NULL. */
        $sql = sprintf('UPDATE horde_datatree SET datatree_data = NULL WHERE datatree_id = %s',
                       (int)$a['id']);

        $result = $db->query($sql);
    }
} else {
    var_dump($attr);
}
