<?php
/**
 * The Agora script to display a list of threads in a forum.
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * $Horde: agora/threads.php,v 1.30 2005/01/03 14:34:46 jan Exp $
 *
 * See the enclosed file COPYING for license information (GPL). If you did not
 * receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

@define('AGORA_BASE', dirname(__FILE__));
require_once AGORA_BASE . '/lib/base.php';
require_once AGORA_BASE . '/lib/Messages.php';
require_once 'Horde/Text/Filter.php';
require_once 'Horde/Variables.php';
require_once 'Horde/UI/Pager.php';

/* Make sure we have a forum id. */
list($forum_id) = Agora::getAgoraId();
if (empty($forum_id)) {
    header('Location: ' . Horde::applicationUrl('forums.php', true));
    exit;
}

/* Check if this is a valid thread, otherwise show the forum list. */
$threads = &Agora_Messages::singleton(array('forum_id' => $forum_id));
if (is_a($threads, 'PEAR_Error')) {
    $notification->push(sprintf(_("Could not list threads. %s"), $threads->getMessage()), 'horde.warning');
    header('Location: ' . Horde::applicationUrl('forums.php', true));
    exit;
}

/* Which thread page are we on?  Default to page 0. */
$thread_page = Util::getFormData('thread_page', 0);
$threads_per_page = $prefs->getValue('threads_per_page');
$thread_start = $thread_page * $threads_per_page;

/* Get the forum data. */
$forum_array = $threads->forum->getForumArray();

/* Get the sorting. */
$sort_by = Agora::getSortBy('threads');
$sort_dir = Agora::getSortDir('threads');

/* Get a list of threads. */
$threads_list = $threads->getThreads($forum_id, false, $sort_by, $sort_dir);
if (empty($threads_list)) {
    $threads_count = 0;
    $notification->push(_("No threads available."), 'horde.warning');
} else {
    $threads_count = count($threads_list);
    $threads_list = array_slice($threads_list, $thread_start, $threads_per_page);
}

/* Set up the column headers. */
$col_headers = array('message_subject' => _("Subject"), 'message_author' => _("Posted By"), 'message_date' => _("Date"));
$col_headers = Agora::formatColumnHeaders($col_headers, $sort_by, $sort_dir, 'threads');

/* Set up the template tags. */
$template = &new Horde_Template();
$template->set('col_headers', $col_headers);
$template->set('threads', $threads_list, true);
$template->set('forum_name', $forum_array['forum_name']);
$template->set('forum_description', Text_Filter::filter($forum_array['forum_description'], 'text2html', array('parselevel' => TEXT_HTML_MICRO)));
$template->set('actions', $threads->getThreadActions(), true);
$template->set('menu', Agora::getMenu('string'));
$template->set('notify', Util::bufferOutput(array($notification, 'notify'), array('listeners' => 'status')));

/* Set the gettext option. */
$template->setOption('gettext', true);

/* Set up pager. */
$vars = &Variables::getDefaultVariables();
$pager_ob = &new Horde_UI_Pager('thread_page', $vars, array('num' => $threads_count, 'url' => 'threads.php', 'perpage' => $threads_per_page));
$pager_ob->preserve('agora', Util::getFormData('agora'));
$template->set('pager_link', $pager_ob->render(), true);

$title = sprintf(_("Threads in %s"), $forum_array['forum_name']);
require AGORA_TEMPLATES . '/common-header.inc';
echo $template->fetch(AGORA_TEMPLATES . '/threads/threads.html');
require $registry->get('templates', 'horde') . '/common-footer.inc';
